/* A simple program to show how to construct and print a Tree */
/* The tree is organized as a "dictionary" (smaller key go to the left,
 * bigger keys go to the right).
*/

class Tree {
  public long info;
  public Tree SX, DX;

  Tree() {	// Tree constructor
   SX = DX = null;
  }	

  Tree(long n) {	// Tree constructor
   info = n;
   SX = DX = null;
  }	

  public int insert(Tree t) {
    if (t != null)  {
	if (this.info <= t.info) { // Insert into right subtree
	    if (this.DX  == null) { // Right subtree is empty
		this.DX = t;
		return 1;
	    } else { // Right subtree NOT empty
		return 1+this.DX.insert(t);
	    }
	} else { // Insert into left subtree
	    if (this.SX  == null) { // Left subtree is empty
		this.SX = t;
		return 1;
	    } else { // Left subtree NOT empty
		return 1+this.SX.insert(t);
	    }
	}
    }
    return 0;
  } // End of insert


// This method prints all Nodes in this Tree (in order: SX, Root, DX)
  public void print(Tree t) { 
    if (t != null) {
	print (t.SX);
	System.out.println("Info:\t" + t.info);
	print (t.DX);
    }
  } // End of print

// An alternative method to print a tree (in order: SX, Root, DX)
// This is more object-oriented than the one above
  public void printTree() { 
    if (this.SX != null) this.SX.printTree();
    System.out.println("Info:\t" + this.info);
    if (this.DX != null) this.DX.printTree();
  } // End of printObject

// A method to print the tree structure and its contents in "graphical form"
  public void printGraphic(int l) { 
    if (this.DX != null) this.DX.printGraphic(l+1);
    for (int i=0; i<l; ++i)
	System.out.print("\t");
    System.out.println(this.info);
    if (this.SX != null) this.SX.printGraphic(l+1);
  } // End of printGraphic

} // End of Tree Class

// This class is here only to provide a main that tests all methods
class testTree {
  public static void main(String args[]) {
    Tree t1;// Define a Tree t1   
    long k;
    k = (long)Math.floor(Math.random()*100);
    t1 = new Tree(k);	// t1 now has only the root
    System.out.println("\nInserting 30 random numbers into  dict. Tree t1");
    System.out.println (k + "(root)\t0 steps");
    for (int n=0; n<30; ++n) {
      k = (long)Math.floor(Math.random()*100);
      System.out.print(k + "\t\t");
      Tree temp = new Tree(k);
      int steps = t1.insert(temp);
      System.out.println(steps + " steps");
   }
    System.out.println("\nPrinting Tree t1");
    t1.print(t1);

    System.out.println("\n\nNow printing again with second method");
    t1.printTree();

    System.out.println("\n\nNow printing again with graphic method");
    t1.printGraphic(0);

    System.out.println("\n\nChecking how many steps are needed to insert n elements");
    System.out.println("n\t\tTotSteps\tAveSteps\tTime");
    for (long n=16; n<1000000L; n*=2) {
	Tree t2;
	k = (long)Math.floor(Math.random()*n);
	t2 = new Tree(k);
	int count = 0; long startTime = System.currentTimeMillis();
	for (long i=0; i<n; ++i) { // inserting n elements into t2
	    k = (long)Math.floor(Math.random()*n);
	    Tree temp = new Tree(k);
	    count += t2.insert(temp);
	}
	long endTime = System.currentTimeMillis();
	double steps = (double)count/(double)n;
	System.out.println(n + "\t\t" + 
			  count + "\t\t" + 
			  steps + "\t\t" + 
			  (long)(endTime-startTime) );
    }
  }
}
